%
%
%  Distance Transformation for Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Surfaces Functions">
%        <Item name="Distance Transformation" icon="Matlab" tooltip="Voxel intensity of new channel indicates the shortest distance to a Surfaces object.">
%          <Command>MatlabXT::XTDistanceTransform(%i)</Command>
%        </Item>
%       </Submenu>
%       <Submenu name="Spots Functions">
%        <Item name="Distance Transformation" icon="Matlab" tooltip="Voxel intensity of new channel indicates the shortest distance to the centers of a Spots object.">
%          <Command>MatlabXT::XTDistanceTransform(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%      <SurpassTab>
%        <SurpassComponent name="bpSurfaces">
%          <Item name="Distance Transformation" icon="Matlab" tooltip="Voxel intensity of new channel indicates the shortest distance to a Surfaces object.">
%            <Command>MatlabXT::XTDistanceTransform(%i)</Command>
%          </Item>
%        </SurpassComponent>
%        <SurpassComponent name="bpSpots">
%          <Item name="Distance Transformation" icon="Matlab" tooltip="Voxel intensity of new channel indicates the shortest distance to the centers of a Spots object.">
%            <Command>MatlabXT::XTDistanceTransform(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
%  
%
%  Description:
%
%   Create and compute a new channel, where the voxel intensity indicates
%       the shortest distance to a Surfaces object, or centers of a Spots object.
%
%

function XTDistanceTransform(aImarisApplicationID)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end
vImarisDataSet = vImarisApplication.GetDataSet.Clone;

% Get Surpass Surfaces Object
vImarisObject = vImarisApplication.GetSurpassSelection;            
% Check if there is a selection
if isempty(vImarisObject)
  msgbox('A spots/surfaces object must be selected');
  return;
end

vIsSurfaces = false;

% Check if the selection is a surfaces object
if vImarisApplication.GetFactory.IsSurfaces(vImarisObject)
  vImarisObject = vImarisApplication.GetFactory.ToSurfaces(vImarisObject);
  vIsSurfaces = true;
elseif vImarisApplication.GetFactory.IsSpots(vImarisObject)
  vImarisObject = vImarisApplication.GetFactory.ToSpots(vImarisObject);
else
  msgbox('Your selection is not a valid spots or surfaces object');
  return;
end

% if the DataSet is not of type float, display a warning. As the result of
% DistanceTransform will very probably be floating values, a DataSet of
% type int will not be able to display it correctly.
if strcmp(vImarisDataSet.GetType,'eTypeUInt8') || strcmp(vImarisDataSet.GetType,'eTypeUInt16')
  vWarningAnswer = warndlg('Due to the DataSet type the results of the Distance Transform function will probably be truncated! It is best to use it with a float DataSet!','!Warning!');
  waitfor(vWarningAnswer);
end

vImarisApplication.DataSetPushUndo('Distance Transform');

vDataMin = [vImarisDataSet.GetExtendMinX, vImarisDataSet.GetExtendMinY, vImarisDataSet.GetExtendMinZ];
vDataMax = [vImarisDataSet.GetExtendMaxX, vImarisDataSet.GetExtendMaxY, vImarisDataSet.GetExtendMaxZ];
vDataSize = [vImarisDataSet.GetSizeX, vImarisDataSet.GetSizeY, vImarisDataSet.GetSizeZ];

vSelection = 1;
vSpotsXYZ = [];
vSpotsTime = [];
if vIsSurfaces
  % Draw the input dialog and ask for a threshold.    
  [vSelection,vOk] = listdlg('ListString',{'Inside SurfaceObject','Outside SurfaceObject'},...
                         'SelectionMode','single',...
                         'ListSize',[200 50],...
                         'Name','XTDistanceTransform - Selection Box',...
                         'PromptString',{'Please select Distance Transform mode:'});
  % Go on if the threshold value exists.
  if vOk < 1, return, end
else 
  vSpotsValues = vImarisObject.Get;
  vSpotsXYZ = vSpotsValues.mPositionsXYZ;
  vSpotsTime = vSpotsValues.mIndicesT;

  vInd = ones(numel(vSpotsTime), 1);
  for vDim = 1:3
    vSpotsXYZ(:, vDim) = floor((vSpotsXYZ(:, vDim) - vDataMin(vDim)) ./ ...
      (vDataMax(vDim) - vDataMin(vDim)) .* vDataSize(vDim)) + 1;
    vInd = vInd & vSpotsXYZ(:, vDim) > 0 & vSpotsXYZ(:, vDim) <= vDataSize(vDim);
  end
  vSpotsXYZ = vSpotsXYZ(vInd, :);
  vSpotsTime = vSpotsTime(vInd);
end

vProgressDisplay = waitbar(0, 'Distance Transform: Preparation');

% Create a new channel where the result will be sent
vNumberOfChannels = vImarisDataSet.GetSizeC;
vImarisDataSet.SetSizeC(vNumberOfChannels + 1);
vImarisDataSet.SetChannelName(vNumberOfChannels,['Distance to ', char(vImarisObject.GetName)]);
vImarisDataSet.SetChannelColorRGBA(vNumberOfChannels, 255*256*256);

vSizeT = vImarisDataSet.GetSizeT;
for vTime = 0:vSizeT-1;
  if vIsSurfaces
    
    % Get the mask DataSet
    vMaskDataSet = vImarisObject.GetMask( ...
      vDataMin(1), vDataMin(2), vDataMin(3), ...
      vDataMax(1), vDataMax(2), vDataMax(3), ...
      vDataSize(1), vDataSize(2), vDataSize(3), vTime);
    for vIndexZ = 1:vDataSize(3)
      vSlice = vMaskDataSet.GetDataSliceBytes(vIndexZ-1, 0, 0);
      if vSelection == 1
        vSlice = vSlice ~= 1;
      else
        vSlice = vSlice == 1;
      end
      if strcmp(vImarisDataSet.GetType,'eTypeUInt8')
        vImarisDataSet.SetDataSliceBytes(vSlice, ...
          vIndexZ-1, vNumberOfChannels, vTime);
      elseif strcmp(vImarisDataSet.GetType,'eTypeUInt16')
        vImarisDataSet.SetDataSliceShorts(vSlice, ...
          vIndexZ-1, vNumberOfChannels, vTime);
      elseif strcmp(vImarisDataSet.GetType,'eTypeFloat')
        vImarisDataSet.SetDataSliceFloats(vSlice, ...
          vIndexZ-1, vNumberOfChannels, vTime);
      end
    end
    
  else % is spots
    
    vThisTime = vSpotsTime == vTime;
    vThisSpots = vSpotsXYZ(vThisTime, :);
    for vIndexZ = 1:vDataSize(3)
      vSliceSpots = vThisSpots(vThisSpots(:, 3) == vIndexZ, :);
      if isempty(vSliceSpots)
        continue;
      end
      vSlice = zeros(vDataSize(1), vDataSize(2), 'uint8');
      vSlice(vSliceSpots(:, 1) + (vSliceSpots(:, 2) - 1) * vDataSize(1)) = 1;
      if strcmp(vImarisDataSet.GetType,'eTypeUInt8')
        vImarisDataSet.SetDataSliceBytes(vSlice, ...
          vIndexZ-1, vNumberOfChannels, vTime);
      elseif strcmp(vImarisDataSet.GetType,'eTypeUInt16')
        vImarisDataSet.SetDataSliceShorts(vSlice, ...
          vIndexZ-1, vNumberOfChannels, vTime);
      elseif strcmp(vImarisDataSet.GetType,'eTypeFloat')
        vImarisDataSet.SetDataSliceFloats(vSlice, ...
          vIndexZ-1, vNumberOfChannels, vTime);
      end
    end
    
  end

  waitbar((vTime+1)/vSizeT/2, vProgressDisplay);
end

waitbar(0.5, vProgressDisplay, 'Distance Transform: Calculation');
vImarisApplication.GetImageProcessing.DistanceTransformChannel( ...
  vImarisDataSet, vNumberOfChannels, 1, false);
waitbar(1, vProgressDisplay);

vImarisApplication.SetDataSet(vImarisDataSet);
close(vProgressDisplay);


